@@ -47,7 +47,7 @@
 
 #define I965_MAX_PROFILES                       20
 #define I965_MAX_ENTRYPOINTS                    5
-#define I965_MAX_CONFIG_ATTRIBUTES              10
+#define I965_MAX_CONFIG_ATTRIBUTES              32
 #define I965_MAX_IMAGE_FORMATS                  10
 #define I965_MAX_SUBPIC_FORMATS                 6
 #define I965_MAX_SUBPIC_SUM                     4
@@ -73,6 +73,10 @@
 #define ENCODER_HIGH_QUALITY      ENCODER_DEFAULT_QUALITY
 #define ENCODER_LOW_QUALITY       2
 
+#define I965_MAX_NUM_ROI_REGIONS                     8
+
+#define ENCODER_LP_QUALITY_RANGE  8
+
 struct i965_surface
 {
     struct object_base *base;
@@ -141,15 +145,9 @@ struct decode_state
 struct encode_state
 {
     struct codec_state_base base;
-    struct buffer_store *seq_param;
-    struct buffer_store *pic_param;
-    struct buffer_store *pic_control;
     struct buffer_store *iq_matrix;
     struct buffer_store *q_matrix;
-    struct buffer_store **slice_params;
     struct buffer_store *huffman_table;
-    int max_slice_params;
-    int num_slice_params;
 
     /* for ext */
     struct buffer_store *seq_param_ext;
@@ -157,6 +155,7 @@ struct encode_state
     struct buffer_store *packed_header_param[5];
     struct buffer_store *packed_header_data[5];
     struct buffer_store **slice_params_ext;
+    struct buffer_store *encmb_map;
     int max_slice_params_ext;
     int num_slice_params_ext;
 
@@ -231,6 +230,9 @@ struct hw_context
                     union codec_state *codec_state,
                     struct hw_context *hw_context);
     void (*destroy)(void *);
+    VAStatus (*get_status)(VADriverContextP ctx,
+                           struct hw_context *hw_context,
+                           void *buffer);
     struct intel_batchbuffer *batch;
 };
 
@@ -289,6 +291,9 @@ struct object_surface
     uint32_t user_disable_tiling : 1;
     uint32_t user_h_stride_set   : 1;
     uint32_t user_v_stride_set   : 1;
+    /* we need clear right and bottom border for NV12.
+     * to avoid encode run to run issue*/
+    uint32_t border_cleared      : 1;
 
     VAGenericID wrapper_surface;
 
@@ -309,6 +314,7 @@ struct object_buffer
     VABufferInfo export_state;
 
     VAGenericID wrapper_buffer;
+    VAContextID context_id;
 };
 
 struct object_image 
@@ -347,6 +353,8 @@ struct i965_filter
     int ring;
 };
 
+struct i965_driver_data;
+
 struct hw_codec_info
 {
     struct hw_context *(*dec_hw_context_init)(VADriverContextP, struct object_config *);
@@ -356,15 +364,26 @@ struct hw_codec_info
     void (*post_processing_context_init)(VADriverContextP, void *, struct intel_batchbuffer *);
     void (*preinit_hw_codec)(VADriverContextP, struct hw_codec_info *);
 
+    /**
+     * Allows HW info to support per-codec max resolution.  If this functor is
+     * not initialized, then @max_width and @max_height will be used as the
+     * default maximum resolution for all codecs on this HW info.
+     */
+    void (*max_resolution)(struct i965_driver_data *, struct object_config *, int *, int *);
+
     int max_width;
     int max_height;
     int min_linear_wpitch;
     int min_linear_hpitch;
 
     unsigned int h264_mvc_dec_profiles;
+    unsigned int vp9_dec_profiles;
+
     unsigned int h264_dec_chroma_formats;
     unsigned int jpeg_dec_chroma_formats;
     unsigned int jpeg_enc_chroma_formats;
+    unsigned int hevc_dec_chroma_formats;
+    unsigned int vp9_dec_chroma_formats;
 
     unsigned int has_mpeg2_decoding:1;
     unsigned int has_mpeg2_encoding:1;
@@ -385,6 +404,13 @@ struct hw_codec_info
     unsigned int has_h264_mvc_encoding:1;
     unsigned int has_hevc_decoding:1;
     unsigned int has_hevc_encoding:1;
+    unsigned int has_hevc10_decoding:1;
+    unsigned int has_vp9_decoding:1;
+    unsigned int has_vpp_p010:1;
+    unsigned int has_lp_h264_encoding:1;
+    unsigned int has_vp9_encoding:1;
+
+    unsigned int lp_h264_brc_mode;
 
     unsigned int num_filters;
     struct i965_filter filters[VAProcFilterCount];
@@ -473,6 +499,7 @@ va_enc_packed_type_to_idx(int packed_type);
 #define CODEC_JPEG      3
 #define CODEC_VP8       4
 #define CODEC_HEVC      5
+#define CODEC_VP9       6
 
 #define H264_DELIMITER0 0x00
 #define H264_DELIMITER1 0x00
@@ -494,9 +521,17 @@ va_enc_packed_type_to_idx(int packed_type);
 
 struct i965_coded_buffer_segment
 {
-    VACodedBufferSegment base;
-    unsigned char mapped;
-    unsigned char codec;
+    union {
+        VACodedBufferSegment base;
+        unsigned char pad0[64];                 /* change the size if sizeof(VACodedBufferSegment) > 64 */
+    };
+
+    unsigned int mapped;
+    unsigned int codec;
+    unsigned int status_support;
+    unsigned int pad1;
+
+    unsigned int codec_private_data[512];       /* Store codec private data, must be 16-bytes aligned */
 };
 
 #define I965_CODEDBUFFER_HEADER_SIZE   ALIGN(sizeof(struct i965_coded_buffer_segment), 0x1000)
