@@ -36,15 +36,38 @@
 #include "i965_structs.h"
 #include "i965_drv_video.h"
 
+#define I965_BRC_NONE	                0
+#define I965_BRC_CBR	                1
+#define I965_BRC_VBR	                2
+#define I965_BRC_CQP	                3
+
+struct intel_roi
+{
+    short left;
+    short right;
+    short top;
+    short bottom;
+
+    char  value;
+};
+
 struct intel_encoder_context
 {
     struct hw_context base;
     int codec;
     VASurfaceID input_yuv_surface;
-    int is_tmp_id;
     unsigned int rate_control_mode;
+    unsigned int quality_level;
+    unsigned int quality_range;
     void *vme_context;
     void *mfc_context;
+    void *enc_priv_state;
+
+    unsigned int is_tmp_id:1;
+    unsigned int low_power_mode:1;
+    unsigned int soft_batch_force:1;
+    unsigned int context_roi:1;
+
     void (*vme_context_destroy)(void *vme_context);
     VAStatus (*vme_pipeline)(VADriverContextP ctx,
                              VAProfile profile,
@@ -57,11 +80,20 @@ struct intel_encoder_context
                              struct intel_encoder_context *encoder_context);
     void (*mfc_brc_prepare)(struct encode_state *encode_state,
                             struct intel_encoder_context *encoder_context);
+
+    VAStatus (*get_status)(VADriverContextP ctx,
+                           struct intel_encoder_context *encoder_context,
+                           struct i965_coded_buffer_segment *coded_buffer_segment);
 };
 
 extern struct hw_context *
 gen75_enc_hw_context_init(VADriverContextP ctx, struct object_config *obj_config);
 
+extern struct hw_context *
+gen8_enc_hw_context_init(VADriverContextP ctx, struct object_config *obj_config);
+
+extern struct hw_context *
+gen9_enc_hw_context_init(VADriverContextP ctx, struct object_config *obj_config);
 #endif	/* _I965_ENCODER_H_ */
 
 
